package com.ywt.mg.models.PrescriptionPushApiRequest;

import com.fasterxml.jackson.annotation.JsonProperty;

import java.io.Serializable;

public class DrugInfo implements Serializable {

    /**
     * 医嘱有效标志, 见附表：
     *
     * 1 V 核实
     * 2 U 作废
     * 3 H On Hold
     * 4 D 停止
     * 5 P 预开
     * 6 E 执行
     * 7 Q In Queue
     * 8 PO Post-Operation Order
     * 9 S Postponed
     * 10 I 未审核
     * 11 IP 处理中
     * 12 C 撤销
     *
     *
     * 备注：1是数字，就是接口传过的数字，第三列是对应的中文，说白，HIS说会传过来1 2 4
     *  1是核实可以发药品的， 2 4是不能发药，要作废
     */
    @JsonProperty("flag")
    private String flag;

    /**
     * 医嘱id,必须为唯一id
     */
    @JsonProperty("adviceId")
    private String adviceId;

    /**
     * 药品编码
     */
    @JsonProperty("drugNumber")
    private String drugNumber;

    /**
     * 药品商品名称
     */
    @JsonProperty("drugName")
    private String drugName;

    /**
     * 最小包装单位数量
     */
    @JsonProperty("quantity")
    private int quantity;

    /**
     * 最小包装单位(盒、瓶等)
     */
    @JsonProperty("minPackagingUnit")
    private String minPackagingUnit;

    /**
     * 使用频率
     */
    @JsonProperty("frequency")
    private String frequency;

    /**
     * 用药天数
     */
    @JsonProperty("daysOfMedication")
    private String daysOfMedication;

    /**
     * 用法
     */
    @JsonProperty("drugUsage")
    private String drugUsage;

    /**
     * 用量
     */
    @JsonProperty("dosage")
    private String dosage;

    /**
     * 规格
     */
    @JsonProperty("drugSpecifications")
    private String drugSpecifications;

    /**
     * 医保类型,见医保类型附表：
     */
    @JsonProperty("medicalInsurance")
    private String medicalInsurance;

    /**
     * 药房类型,见药房类型附表：
     */
    @JsonProperty("pharmacyType")
    private String pharmacyType;

    /**
     * 药品类型,见药品类型附表：
     */
    @JsonProperty("drugType")
    private String drugType;

    /**
     * 备注,如果没有,给空字符串
     */
    @JsonProperty("note")
    private String note;

    /**
     * 医嘱类型,见医嘱类型附表：
     */
    @JsonProperty("doctorAdviceType")
    private String doctorAdviceType;

    public String getFlag() {
        return flag;
    }

    public void setFlag(String flag) {
        this.flag = flag;
    }

    public String getAdviceId() {
        return adviceId;
    }

    public void setAdviceId(String adviceId) {
        this.adviceId = adviceId;
    }

    public String getDrugNumber() {
        return drugNumber;
    }

    public void setDrugNumber(String drugNumber) {
        this.drugNumber = drugNumber;
    }

    public String getDrugName() {
        return drugName;
    }

    public void setDrugName(String drugName) {
        this.drugName = drugName;
    }

    public int getQuantity() {
        return quantity;
    }

    public void setQuantity(int quantity) {
        this.quantity = quantity;
    }

    public String getMinPackagingUnit() {
        return minPackagingUnit;
    }

    public void setMinPackagingUnit(String minPackagingUnit) {
        this.minPackagingUnit = minPackagingUnit;
    }

    public String getFrequency() {
        return frequency;
    }

    public void setFrequency(String frequency) {
        this.frequency = frequency;
    }

    public String getDaysOfMedication() {
        return daysOfMedication;
    }

    public void setDaysOfMedication(String daysOfMedication) {
        this.daysOfMedication = daysOfMedication;
    }

    public String getDrugUsage() {
        return drugUsage;
    }

    public void setDrugUsage(String drugUsage) {
        this.drugUsage = drugUsage;
    }

    public String getDosage() {
        return dosage;
    }

    public void setDosage(String dosage) {
        this.dosage = dosage;
    }

    public String getDrugSpecifications() {
        return drugSpecifications;
    }

    public void setDrugSpecifications(String drugSpecifications) {
        this.drugSpecifications = drugSpecifications;
    }

    public String getMedicalInsurance() {
        return medicalInsurance;
    }

    public void setMedicalInsurance(String medicalInsurance) {
        this.medicalInsurance = medicalInsurance;
    }

    public String getPharmacyType() {
        return pharmacyType;
    }

    public void setPharmacyType(String pharmacyType) {
        this.pharmacyType = pharmacyType;
    }

    public String getDrugType() {
        return drugType;
    }

    public void setDrugType(String drugType) {
        this.drugType = drugType;
    }

    public String getNote() {
        return note;
    }

    public void setNote(String note) {
        this.note = note;
    }

    public String getDoctorAdviceType() {
        return doctorAdviceType;
    }

    public void setDoctorAdviceType(String doctorAdviceType) {
        this.doctorAdviceType = doctorAdviceType;
    }
}
