package com.ywt.mg.web.controller;

import com.ywt.gapi.ResultCode;
import com.ywt.gapi.drug.DrugServiceGrpc;
import com.ywt.gapi.drug.PrescriptionInfo;
import com.ywt.gapi.drug.SaveHisOpenApiPrescriptionRequest;
import com.ywt.gapi.drug.SaveHisOpenApiPrescriptionResponse;
import com.ywt.mg.core.AppMessageException;
import com.ywt.mg.core.JsonSerializer;
import com.ywt.mg.models.BaseResponse;
import com.ywt.mg.models.DataResponse;
import com.ywt.mg.models.PrescriptionPushApiRequest.DoctorInfo;
import com.ywt.mg.models.PrescriptionPushApiRequest.DrugInfo;
import com.ywt.mg.models.PrescriptionPushApiRequest.PatientInfo;
import com.ywt.mg.models.PrescriptionPushApiRequest.PrescriptionPushData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author daiyihua
 * @create 2022-12-05 11:15
 * @program his_open_api
 * @description 太和、白云药品服务
 **/
@RestController("/drug")
@RequestMapping({"/drug"})
public class DrugController {

    private static Logger logger = LoggerFactory.getLogger(DrugController.class);

    @Autowired
    private DrugServiceGrpc.DrugServiceBlockingStub drugServiceBlockingStub;

    /**
     * 查询药品订单列表
     */
    @RequestMapping(value = {"/prescriptionPush"}, method = RequestMethod.POST)
    BaseResponse prescriptionPush(@RequestBody Map<String, String> dataMap) {
        BaseResponse response = new BaseResponse();

        String eventNo = dataMap.getOrDefault("eventNo", "");
        String eventType = dataMap.getOrDefault("eventType", "").trim();
        String eventData = dataMap.getOrDefault("eventData", "");
        logger.info("---eventNo---:{}", eventNo);
        logger.info("---eventType---:{}", eventType);
        logger.info("---eventData---:{}", eventData);

        switch (eventType) {
            case "prescriptionPush":
                return prescriptionPushApi(eventNo, eventData);
            default:
                return response.failedWithAppError("处理失败，尚未实现该方法");
        }
    }

    private BaseResponse prescriptionPushApi(String eventNo, String eventData) {
        DataResponse response = new DataResponse();
        int code = 0;
        String info = "";
        String no = "";
        String platformNo = "";
        try {
            // {"patient":{"age":"51岁3月13天","birthday":"1971-08-04","gender":"1","height":"","idCard":"341127197108046219","idType":"01","name":"蒋圣武","patientCardNumber":"000000381926","patientNumber":"000265640","patientType":"01","phoneNumber":"13672465144","weight":""},"doctor":{"departmentName":"NFMDXK-内分泌代谢科门诊(黄石院区)","doctorNumber":"9441","gender":"","idCard":"","idType":"","name":"黄永锋","phoneNumber":"","title":"2"},"prescriptions":[{"detail":{"allergyHistory":"","diagnosis":"2型糖尿病","diseaseNumbers":"E11.900","prescriptionNumber":"2534653","prescriptionTime":""},"drugs":[{"adviceId":"2441090||2","daysOfMedication":"30天","doctorAdviceType":"3","dosage":"2片","drugName":"(国)盐酸二甲双胍片[0.25g*100]（国基）","drugNumber":"0505040313","drugSpecifications":"0.25g*100","drugType":"01","drugUsage":"口服","flag":"1","frequency":"Tid","medicalInsurance":"","minPackagingUnit":"瓶(100)","note":"","pharmacyType":"01","quantity":2},{"adviceId":"2441090||3","daysOfMedication":"30天","doctorAdviceType":"3","dosage":"50mg","drugName":"阿卡波糖咀嚼片[50mg*10片*6板]","drugNumber":"0505042111","drugSpecifications":"50mg*10片*6板","drugType":"01","drugUsage":"口服(餐时)","flag":"1","frequency":"Tid","medicalInsurance":"","minPackagingUnit":"盒(60)","note":"","pharmacyType":"01","quantity":2},{"adviceId":"2441090||4","daysOfMedication":"15天","doctorAdviceType":"3","dosage":"9g","drugName":"津力达颗粒[9g*9]（国基）","drugNumber":"3899030111","drugSpecifications":"9g*9袋/盒","drugType":"01","drugUsage":"口服","flag":"1","frequency":"Tid","medicalInsurance":"","minPackagingUnit":"盒(9)","note":"","pharmacyType":"01","quantity":5}]}],"hospitalId":12}
            PrescriptionPushData prescriptionPushData = JsonSerializer.from(eventData, PrescriptionPushData.class);
            if (prescriptionPushData == null) {
                throw new AppMessageException("数据格式不正确");
            }
            logger.info("---- prescriptionPushData: " + JsonSerializer.toJson(prescriptionPushData));
            PatientInfo patient = prescriptionPushData.getPatient();
            DoctorInfo doctor = prescriptionPushData.getDoctor();
            List<com.ywt.mg.models.PrescriptionPushApiRequest.PrescriptionInfo> prescriptions = prescriptionPushData.getPrescriptions();
            int hospitalId = prescriptionPushData.getHospitalId();
            List<PrescriptionInfo> prescriptionInfoList = new ArrayList<>();
            for (com.ywt.mg.models.PrescriptionPushApiRequest.PrescriptionInfo prescription : prescriptions) {
                PrescriptionInfo.Builder pInfoBuilder = PrescriptionInfo.newBuilder()
                        .setNo3Th(prescription.getDetail().getPrescriptionNumber())
                        .setDoctorName(doctor.getName())
                        .setPatientName(patient.getName())
                        .setIdCardNo(patient.getIdCard())
                        .setPatientAge(patient.getAge())
                        .setPatientSex(patient.getGender())
                        .setPatientNo(patient.getPatientNumber())
                        .setDiagnose(prescription.getDetail().getDiagnosis())
                        .setDept(doctor.getDepartmentName())
                        .setPatientMobile(patient.getPhoneNumber())
                        .setPrescriptionDate(prescription.getDetail().getPrescriptionTime())
                        .setHosptialId(hospitalId)
                        //表示his 来源
                        .setSource(6);

                for (DrugInfo drugInfo : prescription.getDrugs()) {

                    int daysOfMedicationInt = 0;
                    String daysOfMedication = drugInfo.getDaysOfMedication();
                    daysOfMedication = daysOfMedication.replace("天","");
                    try {
                        daysOfMedicationInt = Integer.parseInt(daysOfMedication);
                    }catch (Exception e){
                        logger.error("--daysOfMedication 格式化出错--:{}", daysOfMedication);
                    }

                    PrescriptionInfo.PrescriptionDrugInfo.Builder prescriptionDrugInfoBuilder = PrescriptionInfo.PrescriptionDrugInfo.newBuilder()
                            .setDrugCode(drugInfo.getDrugNumber())
                            .setDrugCode3Th(drugInfo.getDrugNumber())
                            .setDrugName(drugInfo.getDrugName())
                            .setDrugSpec(drugInfo.getDrugSpecifications())
                            .setOneDosage(0)
                            .setOneDosageUnit("")
                            .setOneDosageDetail(drugInfo.getDosage())
                            .setAmount(drugInfo.getQuantity())
                            .setUnits(drugInfo.getMinPackagingUnit())
                            .setFrequencyCode(drugInfo.getFrequency())
                            .setFrequency("")
                            .setFrequencyQty(0)
                            .setUsageCode("")
                            .setUsage(drugInfo.getDrugUsage())
                            .setQuantity(drugInfo.getQuantity())
                            .setCourse(daysOfMedicationInt)
                            .setPrice(0)
                            .setExecuteDate("")
                            .setInjections(0)
                            .setPackDetail("")
                            .setGroup(0)
                            .setOneDosageDetail("")
                            .setUsageRemarks(drugInfo.getNote())
                            .setDrugPrintName("")
                            .setDrugRefrigeration("")
                            .setAdviceId(drugInfo.getAdviceId())
                            .setAdviceFlag(Integer.parseInt(drugInfo.getFlag()));
                    pInfoBuilder.addDrugs(prescriptionDrugInfoBuilder);
                }

                prescriptionInfoList.add(pInfoBuilder.build());
            }

            SaveHisOpenApiPrescriptionRequest savePrescriptionRequest = SaveHisOpenApiPrescriptionRequest.newBuilder()
                    .setFlag(0)
                    .addAllPrescriptionInfos(prescriptionInfoList).build();

            SaveHisOpenApiPrescriptionResponse savePrescriptionResponse = drugServiceBlockingStub.saveHisOpenApiPrescription(savePrescriptionRequest);
            if (savePrescriptionResponse.getCode() == ResultCode.SUCCEED_VALUE) {
                code = 0;
                no = "";
                platformNo = savePrescriptionResponse.getBizNo();
                Map item = new HashMap<>(16);
                item.put("no", no);
                item.put("platformNo", platformNo);
                response.setData(item);
            } else {
                code = 3;
                info = savePrescriptionResponse.getInfo();
            }
        } catch (Exception e) {
            return response.failedWithAppError("处理失败:"+e.getMessage());
        }
        return response.failed(code, info);
    }

}

