package com.ywt.mg.web.interceptors;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.ywt.mg.configs.ParameterConfigurer;
import com.ywt.mg.core.*;
import com.ywt.mg.core.utils.serializers.JsonSerializer;
import com.ywt.mg.models.BaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by huangguoping.
 */
@Service
public class AuthInterceptor implements HandlerInterceptor {
    private static Logger logger = LoggerFactory.getLogger(AuthInterceptor.class);

    @Autowired
    private ParameterConfigurer parameterConfigurer;

    @Autowired
    private HttpServletRequestFilter httpServletRequestFilter;

    private void toResponse(BaseResponse response, HttpServletResponse httpServletResponse) throws IOException {
        httpServletResponse.addHeader("Content-Type", "application/json;charset=UTF-8");
        httpServletResponse.setDateHeader("Expires", 0);
        // 禁止缓存的指令：https://developer.mozilla.org/zh-CN/docs/Web/HTTP/Headers/Cache-Control#%E7%A6%81%E6%AD%A2%E7%BC%93%E5%AD%98
        httpServletResponse.setHeader("Cache-Control", "no-cache, no-store, must-revalidate");
        httpServletResponse.setHeader("Pragma", "no-cache");
        String responseData = JsonSerializer.toJson(response);
        byte[] responseDataBytes = responseData.getBytes("utf-8");
        httpServletResponse.setContentLength(responseDataBytes.length);
        httpServletResponse.getOutputStream().write(responseDataBytes);
    }

    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o) throws Exception {
        // 不需要验证
        /*
        if (checkSign(httpServletRequest)) {
            return true;
        }
        BaseResponse baseResponse = new BaseResponse();
        baseResponse.setCode(BaseResponse.NO_RIGHT);
        baseResponse.setInfo("权限校验不通过");
        toResponse(baseResponse, httpServletResponse);
        return false;
         */
        return true;
    }

    /**
     * 获取需要校验的appId
     *
     * @return Map 数组
     */
    public Map<String, String> getCheckAppIdMap() {
        Map appIdMap = new HashMap<String, String>();
        String checkAppIdMap = parameterConfigurer.getCheckAppIdMap();
        try {
            if (!Checker.isNone(checkAppIdMap)) {
                List list = JsonSerializer.from(checkAppIdMap, List.class);
                if (!Checker.isNone(list)) {
                    for (Object o : list) {
                        Map<String, Object> map = (Map<String, Object>) o;
                        String appId = (String) map.getOrDefault("appId", "");
                        String appSecret = (String) map.getOrDefault("appSecret", "");
                        if (!Checker.isNone(appId) && !Checker.isNone(appSecret)) {
                            appIdMap.put(appId, appSecret);
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error("ParameterConfigurer#getCheckappIdMap(checkappIdMap{}参数有误,{})", checkAppIdMap, e.getMessage(), e);
        }
        return appIdMap;
    }

    /**
     * 判断是否是合格的来源
     *
     * @return boolean
     */
    /**
     * 验证签名
     *
     * @return
     * @throws UnsupportedEncodingException
     */
    private boolean checkSign(HttpServletRequest httpServletRequest) throws Exception {
        String authorization = httpServletRequest.getHeader("Authorization");
        String appId = httpServletRequest.getHeader("appId");
        String[] arrValue = authorization.split("Bearer ");
        if (arrValue.length > 1 && !Checker.isNone(appId)) {
            String token = arrValue[1];
            Map<String, String> appIdMap = getCheckAppIdMap();
            String appSecret = appIdMap.getOrDefault(appId, "");
            DecodedJWT jwt = JwtUtil.verify(token, appSecret);
            if (!Checker.isNone(jwt)) {
                String payload = jwt.getClaim("checksum").asString();
                String bodyParam = "";
                if (httpServletRequest instanceof HttpServletRequestFilter.RequestWrapper) {
                    HttpServletRequestFilter.RequestWrapper requestWrapper = (HttpServletRequestFilter.RequestWrapper) httpServletRequest;
                    bodyParam = requestWrapper.getBody();
                }
                String bodyMd5CheckSum = MD5CheckSum.md5(bodyParam).toUpperCase();
                if (payload.equals(bodyMd5CheckSum)) {
                    return true;
                }
            }
            return false;
        }
        return false;
    }


    @Override
    public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, Exception e) throws Exception {

    }


}

